@@ -6,12 +6,12 @@ from collections import OrderedDict
 from grsq.debye import Debye
 
 class RDF:
-    ''' RDF Object.
+    ''' Class for representing a pair-RDF sampled between two atom types.
 
         r: np.ndarray (1, )
-            r vector
+            r vector: The r vector defining the bins in which the RDF was sampled, in Å.
         g: np.ndarray, same dim as r
-            g(r) values
+            g(r) values of each r bin
         name1: str
             Name of atomtype/element of 'left' atoms
         name2: str
@@ -20,12 +20,25 @@ class RDF:
             Which region does the 'left' atom belong to
         region2: str, 'solute', or 'solvent'
             Which region does the 'right' atom belong to.
+        n1: int
+            Number of atoms of this type 
+        n2: int
+            Number of atoms of this type
+        damp: grsg.damping.Damping
+            An instance of the `grsq.damping.Damping` class that provides the damping
+            function used in the RDF calculations. See the documentation for
+            :class:`~grsq.damping.Damping` for more information on the methods and attributes
+            available in the Damping class.
         qvec: np.ndarray (1, )
-            Scattering vector
+            Scattering vector, in Å^-1
         rho_norm: str, 'KD' or 'N'
-            Specify which bulk density was used in in obtaining g(r)
-            '(N - KD_ij) / V': As in e.g. VMD
-            'N/V': As in e.g. MDAnalysis
+            | Specify which bulk density was used in in obtaining g(r)
+            | '(N - KD_ij) / V': As in e.g. VMD
+            | 'N/V': As in e.g. MDAnalysis
+        r_max: float
+            Specify at which r to stop the integration
+        r_avg: float
+            Use avg. g(r > r_avg) for g0
     '''
 
     def __init__(self, r, g, name1, name2,
@@ -44,6 +57,9 @@ class RDF:
         self.n1 = n1  # number of 'left' atoms
         self.n2 = n2  # number of 'right' atoms
 
+        self.f0_1 = None  # atomic form factors
+        self.f0_2 = None
+
         self.diagonal = (name1 == name2) & (region1 == region2)
         self.cross = (region1 != region2)
 
@@ -66,6 +82,7 @@ class RDF:
         return self.get_info()
 
     def get_info(self):
+        ''' Show contents of RDF object'''
         info = f'{self.name1}-{self.region1}--{self.name2}-{self.region2}: ' +\
                f'RDF with {self.n1} particles in {self.region1} and {self.n2} in {self.region2}. ' +\
                f'Volume: {self.volume}'
@@ -117,30 +134,38 @@ class RDF:
 
         return s
 
-    def atomic_f0(self):
+    def atomic_f0(self, custom_cm=False):
         ''' Calculate atomic form factors and
             update RDF object in place '''
         from grsq import Debye
-        db = Debye(self.qvec)
+        db = Debye(self.qvec, custom_cm=custom_cm)
         self.f0_1 = db.atomic_f0(self.name1)
         self.f0_2 = db.atomic_f0(self.name2)
 
-    def i_term1(self):
+    def i_term1(self, custom_cm=False):
         ''' Coherent X-Ray Scattering, atomic term'''
         kd = int((self.region1 == self.region2) and (self.name1 == self.name2))
         term_1 = np.zeros(len(self.qvec))
-        self.atomic_f0()
+        if custom_cm: # reset f0s to recalculate with custom_cm
+            self.f0_1 = None
+            self.f0_2 = None
+        if (self.f0_1 is None) or (self.f0_2 is None):
+            self.atomic_f0(custom_cm)
         for n, f0 in zip((self.n1, self.n2), (self.f0_1, self.f0_2)):
             term_1 += (n - kd) * f0**2
         return term_1
 
-    def i_term2(self):
+    def i_term2(self, custom_cm=False):
         ''' Coherent X-Ray Scattering, interatomic term'''
         kd = int((self.region1 == self.region2) and (self.name1 == self.name2))
         s = self.structure_factor()
-
+        
         # Get form factors
-        self.atomic_f0()
+        if custom_cm: # reset f0s to recalculate with custom_cm
+            self.f0_1 = None
+            self.f0_2 = None
+        if (self.f0_1 is None) or (self.f0_2 is None):
+            self.atomic_f0(custom_cm=custom_cm)
 
         term_2 = (s - 1) * (self.n1 - kd) * self.f0_1 * self.f0_2
 
@@ -160,10 +185,11 @@ class RDF:
 
     def perera_correct(self, Ri, r_avg):
         ''' See 10.1016/j.molliq.2010.05.006
+
             Ri: float
-                Radius of exluced volume - paper recommends kappa is 2 x this.
+                | Radius of exluced volume - paper recommends kappa is 2 x this.
             r_avg: float
-                Use avg g(r > r_avg)
+                | Use avg g(r > r_avg)
         '''
         rdf = copy.deepcopy(self)
         kappa = 2 * Ri
@@ -195,6 +221,16 @@ class RDF:
         return g * fac
 
     def correct(self, method='volume', **kwargs):
+        ''' Correct for finite size sampling.
+
+            See 10.1063/5.0164365 for an overview, but please
+            cite the original papers of the method(s) you use also.
+
+            method: str: 'volume', 'perera', or 'vegt'
+                | volume: Uses the volume-correction. See: 10.1063/5.0164365
+                | perera: See 10.1016/j.molliq.2010.05.006
+                | vegt: 10.1021/ct301017q
+        '''
         if method.lower() == 'volume':
             return self.volume_correct(kwargs['Ri'])
         elif method.lower() == 'perera':
@@ -225,7 +261,34 @@ class RDF:
 
 class RDFSet(OrderedDict):
     ''' Container object for a set of RDFs.
-        Basically a Dict with a few more options '''
+
+        For more information, see the :ref:`tutorials-label` page in the project documentation.
+
+        Can be used to calculate the solute-solute, cross, and solvent-solvent
+        scattering terms. Given an ``rdfs = RDFSet(...)``
+
+        .. code-block:: python
+
+            >>> qvec = np.arange(0, 10, 0.01)  # define your q range
+            >>> damp = Damping('lorch', L=25)  # use a damping window
+            >>> I_v = rdfs.get_solvent(qvec=qvec, damp=damp)
+            >>> I_c = rdfs.get_cross(qvec=qvec, damp=damp)
+            >>> I_u = rdfs.get_solute(qvec=qvec)  # no damping for solute
+
+        You can also apply the van der Vegt finite size correction to all
+        (cross and solvent) RDFs:
+
+        .. code-block:: python
+
+            >>> rdfs.vdv_correct()
+
+        And calculate the displaced-volume scattering:
+
+        .. code-block:: python
+
+            >>> rdfs.get_dv(qvec=qvec, damp=damp)
+
+    '''
 
     def __init__(self, *arg, **kwargs):
         self.rdfs = {}
@@ -263,6 +326,12 @@ class RDFSet(OrderedDict):
         else:
             raise Exception("RDF not in set")
 
+    def __setitem__(self, key, rdf):
+        if isinstance(key, tuple) and len(key) == 4:
+            expected_key = (rdf.name1, rdf.region1, rdf.name2, rdf.region2)
+            if key == expected_key:
+                super().__setitem__(key, rdf)
+
     def __add__(self, add):
         tmp = RDFSet()
         for key, rdf in self.items():
@@ -271,6 +340,53 @@ class RDFSet(OrderedDict):
             tmp[(rdf.name1, rdf.region1, rdf.name2, rdf.region2)] = rdf
         return tmp
 
+    def __eq__(self, other):
+        if not isinstance(other, RDFSet):
+            return False
+        if not len(self) == len(other):
+            print(f'Different number of RDFs in sets, {len(self), len(other)}')
+            return False
+        # compare RDF attributes
+        for (key_self, rdf_self), (key_other, rdf_other) in zip(self.items(), other.items()):
+            if key_self != key_other:
+                print(f'{key_self} != {key_other}')
+                return False
+            if not (rdf_self.name1 == rdf_other.name1 and
+                    rdf_self.region1 == rdf_other.region1 and
+                    rdf_self.name2 == rdf_other.name2 and
+                    rdf_self.region2 == rdf_other.region2 and
+                    rdf_self.volume == rdf_other.volume):
+                print('Differences:')
+                print(rdf_self.get_info())
+                print(rdf_other.get_info())
+                return False
+            if not (rdf_self.n1 == rdf_other.n1 and
+                    rdf_self.n2 == rdf_other.n2):
+                print('Different number of atoms:')
+                print(rdf_self.get_info())
+                print(rdf_other.get_info())
+                return False
+            if len(rdf_self.g) != len(rdf_other.g):
+                print('g(r)s have different length')
+                print(rdf_self.get_info())
+                print(rdf_other.get_info())
+                return False
+            if len(rdf_self.r) != len(rdf_other.r):
+                print('r has different length')
+                print(rdf_self.get_info())
+                print(rdf_other.get_info())
+                return False
+            if not (rdf_self.r == rdf_other.r).all():
+                print('g(r) different in')
+                print(rdf_self.get_info())
+                return False
+            if not (rdf_self.g == rdf_other.g).all():
+                print('r different in')
+                print(rdf_self.get_info())
+                return False
+        return True
+
+
     def add_rdf(self, rdf):
         self[(rdf.name1, rdf.region1, rdf.name2, rdf.region2)] = rdf
 
@@ -286,6 +402,7 @@ class RDFSet(OrderedDict):
                    r_rdf.name2, r_rdf.region2)] = r_rdf
 
     def show(self):
+        ''' Show contents of RDFSet. '''
         print(f'|  #  | KEY                     |     STOICHOMETRY     | REGION1, REGION2 | VOLUME ')
         for i, (key, rdf) in enumerate(self.items()):
             key1 = '-'.join(k for k in key[:2])
@@ -295,7 +412,7 @@ class RDFSet(OrderedDict):
                   f'{rdf.region1:7s}, {rdf.region2:7s} | {rdf.volume:6.4f} |')
 
 
-    def get_iq(self, qvec=None, cross=False, damping=None):
+    def get_iq(self, qvec=None, cross=False, damping=None, custom_cm=False):
         ''' Calculate Coherent Scattering from set '''
 
         V = np.unique([rdf.volume for rdf in self.values()])
@@ -312,7 +429,7 @@ class RDFSet(OrderedDict):
 
         icoh = np.zeros(len(qvec))
         if not cross:
-            deb = Debye(qvec=qvec)
+            deb = Debye(qvec=qvec, custom_cm=custom_cm)
             for atm, n in total_atom_types:
                 element = atm.split('-')[0]
                 icoh += n * deb.atomic_f0(element)**2
@@ -322,7 +439,7 @@ class RDFSet(OrderedDict):
             rdf.qvec = qvec
             if damping is not None:
                 rdf.damp = damping
-            icoh += rdf.i_term2()
+            icoh += rdf.i_term2(custom_cm=custom_cm)
         return icoh
 
     def vdv_correct(self):
@@ -335,19 +452,24 @@ class RDFSet(OrderedDict):
                 rdf.g = rdf.vdv_correct()
                 self.add_rdf(rdf)
 
-    def get_solute(self, qvec=None, damping=None):
+    def get_solute(self, qvec=None, damping=None, custom_cm=False):
         ''' Get solute-solute scattering from subset of RDFSet
             It will automatically add Y-X to a set that only has X-Y.
             Which is what you normally want.
+
+            The option to use damping is included for consistency, 
+            but be aware that it does not make much sense to use 
+            it for the solute-solute term, as all RDFs should 
+            go to zero at long r values. 
         '''
         solu = RDFSet()
         for rdf in self.values():
             if (rdf.region1 == 'solute') and (rdf.region2 == 'solute'):
                 solu.add_rdf(copy.deepcopy(rdf))
                 solu.add_flipped(copy.deepcopy(rdf))
-        return solu.get_iq(qvec=qvec, damping=damping)
+        return solu.get_iq(qvec=qvec, damping=damping, custom_cm=custom_cm)
 
-    def get_solvent(self, qvec=None, damping=None):
+    def get_solvent(self, qvec=None, damping=None, custom_cm=False):
         ''' Get solvent-only from subset of RDFSet
             It will automatically add Y-X to a set that only has X-Y.
             Which is what you normally want.
@@ -357,9 +479,9 @@ class RDFSet(OrderedDict):
             if (rdf.region1 == 'solvent') and (rdf.region2 == 'solvent'):
                 solv.add_rdf(copy.deepcopy(rdf))
                 solv.add_flipped(copy.deepcopy(rdf))
-        return solv.get_iq(qvec=qvec, damping=damping)
+        return solv.get_iq(qvec=qvec, damping=damping, custom_cm=custom_cm)
 
-    def get_cross(self, qvec=None, damping=None):
+    def get_cross(self, qvec=None, damping=None, custom_cm=False):
         ''' Get cross-term from subset of RDFSet.
             It will automatically add Y-X to a set that only has X-Y.
             Which is what you normally want. '''
@@ -371,13 +493,16 @@ class RDFSet(OrderedDict):
             elif (rdf.region1 == 'solvent') and (rdf.region2 == 'solute'):
                 solu.add_rdf(copy.deepcopy(rdf))
                 solu.add_flipped(copy.deepcopy(rdf))
-        return solu.get_iq(qvec=qvec, damping=damping, cross=True)
+        return solu.get_iq(qvec=qvec, damping=damping, cross=True, custom_cm=custom_cm)
 
 
-    def get_dv(self, qvec=None, alpha=1, damping=None):
+    def get_dv(self, qvec=None, alpha=1, damping=None, custom_cm=False):
         ''' Displaced Volume Scattering
-        1. Make f_dv form factors for all solute types.
-        2. Calculate total S_dv '''
+
+            1. Make f_dv form factors for all solute types.
+
+            2. Calculate total S_dv
+        '''
         solvent_types = []
         solute_types = []
         for key, rdf in self.items():
@@ -394,7 +519,7 @@ class RDFSet(OrderedDict):
         solvent_types = np.unique(solvent_types)
         solute_types = np.unique(solute_types)
 
-        db = Debye(self[0].qvec)  # use qvec of first RDF. They should all be the same anyway.
+        db = Debye(self[0].qvec, custom_cm=custom_cm)  # use qvec of first RDF. They should all be the same anyway.
         f0_solvs = {}
         for st in solvent_types:
             f0_solvs[st] = db.atomic_f0(st)
@@ -443,30 +568,29 @@ class RDFSet(OrderedDict):
 
         return i_dv
 
-
-## Helper functions
+''' Helper Functions'''
 def rdfset_from_dir(dirname, prmtop=None, volume=None, stoich=None):
     ''' Read in a directory of rdf files with the naming convention:
             gEl1_w1-El2_w2.dat
         where:
-            El1: Element name 1 (also used for the IAM)
-            El2: Element name 2 (also used for the IAM)
-            w1:  Region of element 1 (u, c, v) (i.e. solute, cross, solvent)
-            w2:  Region of element 2 (u, c, v)
+            | El1: Element name 1 (also used for the IAM)
+            | El2: Element name 2 (also used for the IAM)
+            | w1:  Region of element 1 (u, v) (i.e. solUte, solVent)
+            | w2:  Region of element 2 (u, v)
 
             So e.g. water would have the RDFs: gO_v-O_v.dat, gO_v-H_v.dat, etc..
 
-            prmtop (str): path to structure file for a single frame:
-            The method can read the stoichometry from either a prmtop or an xyzfile.
-            If an xyz file is used, you need to specify the volume in which the RDFs
-            where sampled manually.
+        prmtop (str): path to structure file for a single frame:
+            | The method can read the stoichometry from either a prmtop or an xyzfile.
+            | If an xyz file is used, you need to specify the volume in which the RDFs
+            | where sampled manually.
 
-            volume (float): RDF sampling volume. Only use if xyz is specified in prmtop
+        volume (float): RDF sampling volume. Only use if xyz is specified in prmtop
 
-            stoich (dict):
-                {'El1_w1':N1, 'El2_w2:N2 ...}
-            dictionary containing number of atoms of each element/atom type.
-            Use if no prmtop/xyz is used at all
+        stoich (dict):
+            | {'El1_w1':N1, 'El2_w2:N2 ...}
+            | Dictionary containing number of atoms of each element/atom type.
+            | Use if no prmtop/xyz is used at all
         '''
     import MDAnalysis as mda
     import glob
@@ -474,7 +598,7 @@ def rdfset_from_dir(dirname, prmtop=None, volume=None, stoich=None):
     rdf_files = sorted(glob.glob(dirname + os.sep + '*dat'))
     assert len(rdf_files) > 0, f'Found no .dat files in {dirname + os.sep} '
 
-    whered = {'u':'solute', 'v':'solvent', 'c':'cross'}
+    whered = {'u':'solute', 'v':'solvent'}
     if prmtop is not None:
         # Load universe to get stoichometry.
         u = mda.Universe(prmtop)
